#include "stm32f10x_spi.h"
#include "ST7796S.h"

extern volatile uint32_t timerlcd;
lcd_dev lcddev;

//Wpisanie kierunku wpisywania na wyswietlacz
void LCD_direction(uint8_t direction)
{


	//Skonfigurowanie struktury
	lcddev.setxcmd=0x2A;
	lcddev.setycmd=0x2B;
	lcddev.wramcmd=0x2C;
	switch(direction){
		case 0:
			lcddev.width=LCD_W;
			lcddev.height=LCD_H;
			LCD_WriteReg(0x36,(1<<3)|(1<<6));
		break;
		case 1:
			lcddev.width=LCD_H;
			lcddev.height=LCD_W;
			LCD_WriteReg(0x36,(1<<3)|(1<<5));
		break;
		case 2:
			lcddev.width=LCD_W;
			lcddev.height=LCD_H;
			LCD_WriteReg(0x36,(1<<3)|(1<<7));
		break;
		case 3:
			lcddev.width=LCD_H;
			lcddev.height=LCD_W;
			LCD_WriteReg(0x36,(1<<3)|(1<<7)|(1<<6)|(1<<5));
		break;
		default:break;
	}
}

//Ustawienie kursora na punkt
void LCD_SetCursor(uint16_t Xpos, uint16_t Ypos)
{
	LCD_SetWindows(Xpos,Ypos,Xpos,Ypos);
}

//Funkcja ustawiajaca aktywny obszar
//0 - obrot o 0 stopni
//1 - obrot o 90stpni
//2 - obrot o 180 stopni
//3 - obrot o 270 stopni
void LCD_SetWindows(uint16_t xStar, uint16_t yStar,uint16_t xEnd,uint16_t yEnd)
{


	LCD_WR_REG(lcddev.setxcmd);
	LCD_WR_DATA(xStar>>8);
	LCD_WR_DATA(0x00FF&xStar);
	LCD_WR_DATA(xEnd>>8);
	LCD_WR_DATA(0x00FF&xEnd);

	LCD_WR_REG(lcddev.setycmd);
	LCD_WR_DATA(yStar>>8);
	LCD_WR_DATA(0x00FF&yStar);
	LCD_WR_DATA(yEnd>>8);
	LCD_WR_DATA(0x00FF&yEnd);

	LCD_WriteRAM_Prepare();	//ustawienie wpisywania do danych
}


void LCD_Init(void)
{
 	LCD_RESET(); //Reset wyswietlacza
    //Ustawienie parametrow zgodnych z ST7796S
	LCD_WR_REG(0xF0);
	LCD_WR_DATA(0xC3);
	LCD_WR_REG(0xF0);
	LCD_WR_DATA(0x96);
	LCD_WR_REG(0x36);
	LCD_WR_DATA(0x68);
	LCD_WR_REG(0x3A);
	LCD_WR_DATA(0x05);
	LCD_WR_REG(0xB0);
	LCD_WR_DATA(0x80);
	LCD_WR_REG(0xB6);
	LCD_WR_DATA(0x00);
	LCD_WR_DATA(0x02);
	LCD_WR_REG(0xB5);
	LCD_WR_DATA(0x02);
	LCD_WR_DATA(0x03);
	LCD_WR_DATA(0x00);
	LCD_WR_DATA(0x04);
	LCD_WR_REG(0xB1);
	LCD_WR_DATA(0x80);
	LCD_WR_DATA(0x10);
	LCD_WR_REG(0xB4);
	LCD_WR_DATA(0x00);
	LCD_WR_REG(0xB7);
	LCD_WR_DATA(0xC6);
	LCD_WR_REG(0xC5);
	LCD_WR_DATA(0x24);
	LCD_WR_REG(0xE4);
	LCD_WR_DATA(0x31);
	LCD_WR_REG(0xE8);
	LCD_WR_DATA(0x40);
	LCD_WR_DATA(0x8A);
	LCD_WR_DATA(0x00);
	LCD_WR_DATA(0x00);
	LCD_WR_DATA(0x29);
	LCD_WR_DATA(0x19);
	LCD_WR_DATA(0xA5);
	LCD_WR_DATA(0x33);
	LCD_WR_REG(0xC2);
	LCD_WR_REG(0xA7);

	LCD_WR_REG(0xE0);
	LCD_WR_DATA(0xF0);
	LCD_WR_DATA(0x09);
	LCD_WR_DATA(0x13);
	LCD_WR_DATA(0x12);
	LCD_WR_DATA(0x12);
	LCD_WR_DATA(0x2B);
	LCD_WR_DATA(0x3C);
	LCD_WR_DATA(0x44);
	LCD_WR_DATA(0x4B);
	LCD_WR_DATA(0x1B);
	LCD_WR_DATA(0x18);
	LCD_WR_DATA(0x17);
	LCD_WR_DATA(0x1D);
	LCD_WR_DATA(0x21);

	LCD_WR_REG(0XE1);
	LCD_WR_DATA(0xF0);
	LCD_WR_DATA(0x09);
	LCD_WR_DATA(0x13);
	LCD_WR_DATA(0x0C);
	LCD_WR_DATA(0x0D);
	LCD_WR_DATA(0x27);
	LCD_WR_DATA(0x3B);
	LCD_WR_DATA(0x44);
	LCD_WR_DATA(0x4D);
	LCD_WR_DATA(0x0B);
	LCD_WR_DATA(0x17);
	LCD_WR_DATA(0x17);
	LCD_WR_DATA(0x1D);
	LCD_WR_DATA(0x21);

    LCD_WR_REG(0X36);
	LCD_WR_DATA(0xEC);
	LCD_WR_REG(0xF0);
	LCD_WR_DATA(0xC3);
	LCD_WR_REG(0xF0);
	LCD_WR_DATA(0x69);
	LCD_WR_REG(0X13);
	LCD_WR_REG(0X11);
	LCD_WR_REG(0X29);

    LCD_direction(1);    //ustawienie kuerunku wyswietlania
    LCD_LED_ON;                      //zalaczenie podswietleni
	LCD_Clear(White);               //wyczyszczenie obszaru bialym tlem
}

//Funkcja resetujaca kontroler wyswietlacza - ST7796S
void LCD_RESET(void)
{
	LCD_RST_CLR;
	delay_ms(100);
	LCD_RST_SET;
	delay_ms(50);
}

//Funkcja opozniajaca
void delay_ms(uint32_t ms)
{
	timerlcd=ms*10;
	while(timerlcd)
	{

	}

}

//Funkcja rysujaca pixel
void LCD_DrawPoint(uint16_t x,uint16_t y, uint16_t color)
{
	LCD_SetCursor(x,y);//ustawienie kursora
	Lcd_WriteData_16Bit(color);
}

//Wyskanie danej 16biowej
void Lcd_WriteData_16Bit(uint16_t Data)
{
	 LCD_CS_CLR;
	 LCD_RS_SET;
	 SPI_I2S_SendData(SPI1,Data>>8);
	 SPI_I2S_SendData(SPI1,Data);
	 LCD_CS_SET;
}

//Funkcja wysylajaca komende - ze bedziey wpisywac do pamieci ram - bufora ramki
void LCD_WriteRAM_Prepare(void)
{
	LCD_WR_REG(lcddev.wramcmd);
}

//Wyslanie komendy - adresu rejestru
void LCD_WR_REG(uint8_t data)
{
   LCD_CS_CLR;
   LCD_RS_CLR;
   SPI_I2S_SendData(SPI1,data);
   LCD_CS_SET;
}

//wyslanie danej
void LCD_WR_DATA(uint8_t data)
{
   LCD_CS_CLR;
   LCD_RS_SET;
   SPI_I2S_SendData(SPI1,data);
   LCD_CS_SET;
}

//Wyczyszczenie wyswietlacza danym kolorem
void LCD_Clear(uint16_t Color)
{
  unsigned int i,m;
	LCD_SetWindows(0,0,lcddev.width-1,lcddev.height-1);
	LCD_CS_CLR;
	LCD_RS_SET;
	for(i=0;i<lcddev.height;i++)
	{
    for(m=0;m<lcddev.width;m++)
    {
			Lcd_WriteData_16Bit(Color);
		}
	}
	 LCD_CS_SET;
}

//Wpisanie danej do rejestru 16 bitowej
void LCD_WriteReg(uint8_t LCD_Reg, uint16_t LCD_RegValue)
{
	LCD_WR_REG(LCD_Reg);
	LCD_WR_DATA(LCD_RegValue);
}


//rysowanie linii
void tft_line(int X1, int Y1,int X2,int Y2,int Color)
	{
	int CurrentX, CurrentY, Xinc, Yinc,
		Dx, Dy, TwoDx, TwoDy,
		TwoDxAccumulatedError, TwoDyAccumulatedError;

	Dx = (X2-X1);
	Dy = (Y2-Y1);

	TwoDx = Dx + Dx;
	TwoDy = Dy + Dy;

	CurrentX = X1;
	CurrentY = Y1;

	Xinc = 1;
	Yinc = 1;

	if(Dx < 0) {

		Xinc = -1;
		Dx = -Dx;
		TwoDx = -TwoDx;
	}

	if (Dy < 0) {
		Yinc = -1;
		Dy = -Dy;
		TwoDy = -TwoDy;
	}
	LCD_DrawPoint(X1,Y1, Color);


	if ((Dx != 0) || (Dy != 0)) {

		if (Dy <= Dx) {
			TwoDxAccumulatedError = 0;
			do {
			    CurrentX += Xinc;
			    TwoDxAccumulatedError += TwoDy;
			    if(TwoDxAccumulatedError > Dx) {
			        CurrentY += Yinc;
			        TwoDxAccumulatedError -= TwoDx;
			    }

					LCD_DrawPoint(CurrentX,CurrentY, Color);


			  } while (CurrentX != X2);
		  } else {
			  TwoDyAccumulatedError = 0;
			  do {
			      CurrentY += Yinc;
			      TwoDyAccumulatedError += TwoDx;
			      if(TwoDyAccumulatedError>Dy) {
				      CurrentX += Xinc;
				      TwoDyAccumulatedError -= TwoDy;
			      }

						LCD_DrawPoint(CurrentX,CurrentY, Color);


			  } while (CurrentY != Y2);
		  }
	}
}


//Narysowanie obrazka z pamieci flash - powiekszony 2 krotnie
void putpicturesscale(uint16_t x, uint16_t y, uint16_t width, uint16_t height, unsigned char const *grafika)
{
		uint16_t i, j;
		unsigned char const *s, *wsk;
		unsigned char msb, lsb;


		s=grafika;


		for(j=0; j<height; j++)
		{

            LCD_SetWindows(x, y+j*2,x+(width*2)-1,y+(j*2) + 2);

            LCD_CS_CLR;
            asm("NOP");
            LCD_RS_SET;
            wsk=s;
				for(i=0; i<width; i++)
				{
					lsb=*s;
					s++;
					msb=*s;
					s++;
					SPI_I2S_SendData(SPI1,msb);    //format 16bitowy 565 -wyslanie starszej czesci
					SPI_I2S_SendData(SPI1,lsb);     //wyslanie mlodszej czesci
					SPI_I2S_SendData(SPI1,msb);    //format 16bitowy 565 -wyslanie starszej czesci
					SPI_I2S_SendData(SPI1,lsb);     //wyslanie mlodszej czesci
				}
			s=wsk;
			for(i=0; i<width; i++)
					{
								lsb=*s;
								s++;
								msb=*s;
								s++;
								SPI_I2S_SendData(SPI1,msb);    //format 16bitowy 565 -wyslanie starszej czesci
								SPI_I2S_SendData(SPI1,lsb);     //wyslanie mlodszej czesci
								SPI_I2S_SendData(SPI1,msb);    //format 16bitowy 565 -wyslanie starszej czesci
								SPI_I2S_SendData(SPI1,lsb);     //wyslanie mlodszej czesci
					}

			asm("NOP");
		    LCD_CS_SET;
		    asm("NOP");
		}
}

//Funkcja rysujaca tlo o rozdzielczosci 240x160 przeskalowane 2:1
void putscaledwallpaper(unsigned char const *grafika)
{
	putpicturesscale(0, 0, 240, 160, grafika);
}


//Narysowanie obrazka z pamieci flash
void putpictures(uint16_t x, uint16_t y, uint16_t width, uint16_t height, unsigned char const *grafika)
{
		uint16_t i;
		unsigned char const *s;
		unsigned char msb, lsb;
		uint32_t temptemp;

		s=grafika;


	       temptemp=(width+1)*(height+1);

		   LCD_SetWindows(x, y,x+width-1,y+height-1);

		   LCD_CS_CLR;
		   asm("NOP");
		   LCD_RS_SET;


				for(i=0; i<temptemp; i++)
				{
					lsb=*s;
					s++;
					msb=*s;
					s++;
					SPI_I2S_SendData(SPI1,msb);    //format 16bitowy 565 -wyslanie starszej czesci
					SPI_I2S_SendData(SPI1,lsb);     //wyslanie mlodszej czesci
				}

			asm("NOP");
		    LCD_CS_SET;

}




//rysowanie pelnego kwadratu
void full_rect( int X1, int Y1, int X2, int Y2, uint16_t color)
{
	uint16_t temp1, temp2;
	uint32_t tempmul;
	uint32_t k;

	temp1=X2-X1;
	temp2=Y2-Y1;

	tempmul=(temp1+1)*(temp2+1);

	LCD_SetWindows(X1, Y1,X2,Y2);

	LCD_CS_CLR;
	asm("NOP");
	LCD_RS_SET;

	for(k=0; k<=tempmul; k++)
	{
		SPI_I2S_SendData(SPI1,color >> 8);    //format 16bitowy 565 -wyslanie starszej czesci
		SPI_I2S_SendData(SPI1,color & 0xFF);     //wyslanie mlodszej czesci
	}

	asm("NOP");
	 LCD_CS_SET;

}

//Funkcja rysujaca kwadrat z tapety tla - alpha nadaje efekt przezroczystosci
void putrectfromwallpaperscaled(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, unsigned char const *grafika, uint8_t alphacolor)
{
	uint32_t temp;
	uint16_t deltax, deltay;
	unsigned char const *s, *wsktemp;
	unsigned char msb, lsb;
	uint16_t tempcolor, tempred, tempblue, tempgreen;
	uint8_t flagx, flagy;

	deltax=x2-x1;
	deltay=y2-y1;

	flagx=deltax%2;
	flagy=deltay%2;

	if(deltax%2) deltax=deltax-1;
	if(deltay%2) deltay=deltay-1;

	deltay=deltay/2;
	deltax=deltax/2;


	for(uint16_t i=0; i<=deltay; i++)
	{

		temp=(480*((y1/2)+i))+2*(x1/2);
		s=grafika;
		s=s+temp;

		LCD_SetWindows(x1, y1+(i*2),x2 - flagx +1 ,y1 +(i*2) + 2 - flagy);

		asm("NOP");
		LCD_CS_CLR;
		asm("NOP");
		asm("NOP");
		LCD_RS_SET;
		asm("NOP");
		asm("NOP");

		wsktemp=s;
		for(uint16_t z=0; z<2; z++)
		{
			s=wsktemp;
		for(uint16_t k=0; k<=deltax; k++)
		{
			lsb=*s;
			s++;
			msb=*s;
			s++;

			if(alphacolor)
			{
				tempcolor=(msb<<8) | lsb;
				tempred=(tempcolor>>11) & 0x1f;
				tempblue=tempcolor & 0x1f;
				tempgreen=(tempcolor>>5) & 0x3f;
				tempred+=alphacolor;
				if(tempred>31) tempred=31;
				tempred=tempred & 0x1f;
				tempgreen+=(alphacolor);
				if(tempgreen>63) tempgreen=63;
				tempgreen= tempgreen & 0x3f;
				tempblue+=alphacolor;
				if(tempblue>31) tempblue=31;
				tempblue= tempblue & 0x1f;

				tempcolor=(tempred<<11) | (tempgreen<<5) | tempblue;
				msb=tempcolor>>8;
				lsb=tempcolor & 0xff;
			}

			asm("NOP");
			SPI_I2S_SendData(SPI1,msb);        //format 16bitowy 565 -wyslanie starszej czesci
			asm("NOP");
			SPI_I2S_SendData(SPI1,lsb);        //format 16bitowy 565 -wyslanie mlodszej czesci
			asm("NOP");
			SPI_I2S_SendData(SPI1,msb);        //format 16bitowy 565 -wyslanie starszej czesci
			asm("NOP");
			SPI_I2S_SendData(SPI1,lsb);        //format 16bitowy 565 -wyslanie mlodszej czesci
			asm("NOP");
		}
		}

		asm("NOP");
	    LCD_CS_SET;
	    asm("NOP");
	    asm("NOP");

	}
}


//Funkcja rysujaca kwadrat z efekte przezroczystosci wedlug danej funkcji - alphared, alphagreen, alphablue - okreslaja moc koloru przezroczystosci
//subsum - 1 - zwiekszenie danego koloru 0 - zmniejszenie danego koloru
void transparentfromwallpaper(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, unsigned char const *grafika, uint8_t alphared, uint8_t alphagreen, uint8_t alphablue, uint8_t subsum)
{
	uint32_t temp;
	uint16_t deltax, deltay;
	unsigned char const *s, *wsktemp;
	unsigned char msb, lsb;
	uint16_t tempcolor, tempred, tempblue, tempgreen;
	uint8_t flagx, flagy;

	deltax=x2-x1;
	deltay=y2-y1;

	flagx=deltax%2;
	flagy=deltay%2;

	if(deltax%2) deltax=deltax-1;
	if(deltay%2) deltay=deltay-1;

	deltay=deltay/2;
	deltax=deltax/2;


	for(uint16_t i=0; i<=deltay; i++)
	{

		temp=(480*((y1/2)+i))+2*(x1/2);
		s=grafika;
		s=s+temp;

		LCD_SetWindows(x1, y1+(i*2),x2 - flagx +1 ,y1 +(i*2) + 2 - flagy);

		asm("NOP");
		LCD_CS_CLR;
		asm("NOP");
		asm("NOP");
		LCD_RS_SET;
		asm("NOP");
		asm("NOP");

		wsktemp=s;
		for(uint16_t z=0; z<2; z++)
		{
			s=wsktemp;
		for(uint16_t k=0; k<=deltax; k++)
		{
			lsb=*s;
			s++;
			msb=*s;
			s++;

			if(alphared || alphablue || alphagreen)
			{
				tempcolor=(msb<<8) | lsb;
				tempred=(tempcolor>>11) & 0x1f;
				tempblue=tempcolor & 0x1f;
				tempgreen=(tempcolor>>5) & 0x3f;
				if(subsum)
				{
					tempred+=alphared;
					if(tempred>31) tempred=31;
					tempred=tempred & 0x1f;
				}
				else
				{
					tempred-=alphared;
					if(tempred>255) tempred=0;
					tempred=tempred & 0x1f;
				}
				if(subsum)
				{
					tempgreen+=(alphagreen);
					if(tempgreen>63) tempgreen=63;
					tempgreen= tempgreen & 0x3f;
				}
				else
				{
					tempgreen-=(alphagreen);
					if(tempgreen>255) tempgreen=0;
					tempgreen= tempgreen & 0x3f;
				}
				if(subsum)
				{
					tempblue+=alphablue;
					if(tempblue>31) tempblue=31;
					tempblue= tempblue & 0x1f;
				}
				else
				{
					tempblue-=alphablue;
					if(tempblue>255) tempblue=0;
					tempblue= tempblue & 0x1f;
				}


				tempcolor=(tempred<<11) | (tempgreen<<5) | tempblue;
				msb=tempcolor>>8;
				lsb=tempcolor & 0xff;
			}

			asm("NOP");
			SPI_I2S_SendData(SPI1,msb);        //format 16bitowy 565 -wyslanie starszej czesci
			asm("NOP");
			SPI_I2S_SendData(SPI1,lsb);        //format 16bitowy 565 -wyslanie mlodszej czesci
			asm("NOP");
			SPI_I2S_SendData(SPI1,msb);        //format 16bitowy 565 -wyslanie starszej czesci
			asm("NOP");
			SPI_I2S_SendData(SPI1,lsb);        //format 16bitowy 565 -wyslanie mlodszej czesci
			asm("NOP");
		}
		}

		asm("NOP");
	    LCD_CS_SET;
	    asm("NOP");
	    asm("NOP");

	}
}

//Funkcja rysujaca pixel z grafiki
void putpixelfromwallpaper(uint16_t x, uint16_t y, unsigned char const *grafika)
{
	uint32_t temp;

	unsigned char const *s;
	unsigned char msb, lsb;
    s=grafika;


	temp=(480*(y/2)) + (x/2)*2;
	s=s+temp;

	lsb=*s;
	s++;
	msb=*s;


	LCD_SetWindows(x, y,x ,y);

		asm("NOP");
		LCD_CS_CLR;
		asm("NOP");
		asm("NOP");
		LCD_RS_SET;
		asm("NOP");
		asm("NOP");

		SPI_I2S_SendData(SPI1,msb);        //format 16bitowy 565 -wyslanie starszej czesci
		asm("NOP");
		SPI_I2S_SendData(SPI1,lsb);        //format 16bitowy 565 -wyslanie mlodszej czesci

		asm("NOP");
		LCD_CS_SET;
		    asm("NOP");
		    asm("NOP");

}

//Funkcja rysujaca narysowana grafike w monohromatyczna w pikselach
//w tablicy sa zawarte informacje o pikselach - wierszowo
uint8_t putmonograph(uint16_t x, uint16_t y, uint16_t color, uint16_t bkcolor, uint16_t width, uint16_t height,const uint8_t *s)
{
	uint8_t tempi, tempk, wier, dtemp=0, fault=0;
	unsigned char wartosc;
	const uint8_t *p;

	//jezeli wysokosc i szerokosc nie jest podzielna przez 8 to zwroc blad
	if(height%8) fault= 1;
	if(width%8) fault= 1;

	wier=width/8;

	for(tempk=0; tempk<height; tempk++)
	{
	for(tempi=0; tempi<wier; tempi++)
	{
		p=s+(tempk*wier)+tempi;
		wartosc=*p;
		for(dtemp=0; dtemp<8; dtemp++)
		{
			if(wartosc & (1<<(7-dtemp)))
				 LCD_DrawPoint(x+(tempi*8)+dtemp,y+tempk, color);
			else if(bkcolor)  LCD_DrawPoint(x+(tempi*8)+dtemp,y+tempk, bkcolor);
		}

	}
	}

	return fault;
}


//Funkcja rysujaca pojedynczy zaokraglony kwadrat
void tft_round_rect1(int x, int y, int width, int height, uint8_t radius, uint8_t fill, uint16_t color)
{
  	int16_t tSwitch;

  	int x1 = 0, y1 = radius;
  	tSwitch = 3 - 2 * radius;

	while (x1 <= y1)
	{
		 LCD_DrawPoint(x+radius - x1, y+radius - y1, color);
		 LCD_DrawPoint(x+radius - y1, y+radius - x1, color);
		 LCD_DrawPoint(x+width-radius + x1, y+radius - y1 , color);
		 LCD_DrawPoint(x+width-radius + y1,  y+radius - x1, color);
		 LCD_DrawPoint( x+width-radius + x1, y+height-radius + y1, color);
	     LCD_DrawPoint( x+width-radius + y1, y+height-radius + x1, color);
		 LCD_DrawPoint(x+radius - x1, y+height-radius + y1, color);
		 LCD_DrawPoint(x+radius - y1, y+height-radius + x1, color);


			tft_line(x+radius - x1, y+radius - y1, x+width-radius + x1, y+radius - y1, color);
			tft_line(x+radius - y1, y+radius - x1, x+width-radius + y1, y+radius - x1 ,color);
			tft_line(x+width-radius + x1, y+height-radius + y1, x+radius - x1, y+height-radius + y1, color);
			tft_line(x+width-radius + y1, y+height-radius + x1, x+radius - y1, y+height-radius + x1, color);


	    if (tSwitch < 0) {
	    	tSwitch += (4 * x1 + 6);
	    } else {
	    	tSwitch += (4 * (x1 - y1) + 10);
	    	y1--;
	    }
	    x1++;
	}

		uint16_t i;
		for(i=0; i<(height-(2*radius)); i++) tft_line(x, y+radius+i, x+width, y+radius+i, color);

	tft_line(x+radius, y, x+width-radius, y, color);
	tft_line(x+radius, y+height, x+width-radius, y+height, color);

	tft_line(x, y+radius, x, y+height-radius, color);
	tft_line(x+width, y+radius, x+width, y+height-radius, color);
}

//Kwadrat z zaokraglonynmi krawedziami
void tft_round_rect(int x, int y, int width, int height, uint8_t radius, uint16_t color1, uint16_t color2)
{
		tft_round_rect1(x,y,width,height, radius,1, color1);
		tft_round_rect1(x+2,y+2,width-4,height-4, radius,1, color2);
}
