#include "stm32f10x.h"
#include "graphics.h"
#include "monographics.h"

/* Opis funkcjonalny
 * PC0 - ADC10 - current
 * PC1 - ADC11
 * PC2 - ADC12
 * PC3 - ADC13
 *
 * PA0 - ADC0 - voltage
 * PA1 - ADC1
 * PA2 - ADC2
 * PA3 - ADC3
 *
 * PC6, PC7, PC8, PC9 - porty kontrolujace czy ogniowo jest ladowane czy rozladowywane
*/

//******************************************************************
//Definicje konfiguracyjne
#define refvoltage 300
#define refcurrent 400
#define supvoltage 143
#define infvoltage 106
#define interval2 5000
#define interval 10000  //sprawdzenie co 1s
#define interval3 9000
#define aproxch 10800   //maksymalny czas ladowania powyzej napiecia 1.43 - 3h
#define aproxuch 66     //1m czas rozladowania ponizej 1,03V
#define endofch  10
#define supersup 154
#define supersup2 170
#define quiescent 5
#define hysteresis 5
#define hysteresis2 25
#define hysteresis3 10
#define compvoltage 200
#define compcurrent 20
#define variabend   1500 //odczekanie 25minut

//******************************************************************
#define valset1 TIM4->CCR1
#define valset2 TIM4->CCR2
#define valset3 TIM4->CCR3
#define valset4 TIM4->CCR4
#define backlight  TIM3->CCR4
#define docalibration !(GPIOB->IDR & GPIO_Pin_0)  //definicja aktywowania kalibracji
#define charge1  GPIOC->BSRR=GPIO_Pin_6
#define charge2  GPIOC->BSRR=GPIO_Pin_7
#define charge3  GPIOC->BSRR=GPIO_Pin_8
#define charge4  GPIOC->BSRR=GPIO_Pin_9
#define uncharge1  GPIOC->BRR=GPIO_Pin_6
#define uncharge2  GPIOC->BRR=GPIO_Pin_7
#define uncharge3  GPIOC->BRR=GPIO_Pin_8
#define uncharge4  GPIOC->BRR=GPIO_Pin_9

#define ledinfo1on  GPIOC->BSRR=GPIO_Pin_10
#define ledinfo2on  GPIOC->BSRR=GPIO_Pin_11
#define ledinfo3on  GPIOC->BSRR=GPIO_Pin_12
#define ledinfo4on  GPIOD->BSRR=GPIO_Pin_2
#define ledinfo1off GPIOC->BRR=GPIO_Pin_10
#define ledinfo2off GPIOC->BRR=GPIO_Pin_11
#define ledinfo3off GPIOC->BRR=GPIO_Pin_12
#define ledinfo4off GPIOD->BRR=GPIO_Pin_2

//******************************************************************
//Funkcje systemowe
void GPIO_Config(void);
void RCC_Config(void);
void NVIC_Config(void);
void SPI_init(void);
void ADC_Config(void);
void DMA_Config(void);
void TIM_Config(void);
//******************************************************************
//Funkcje uytkowe
uint8_t gettouchvalue(void);
void calibrationtouch(void);
void animewallpaper(void);
void animwallpaper2(void);
void barbattery(uint16_t *value, uint16_t *valuetemp, uint8_t *flagapomoc, uint16_t x, uint16_t y);
void setcurrent(uint16_t *value, uint16_t *valuetemp, uint16_t x, uint16_t y);
void touchcontrol(void);
void putbuttons(uint16_t x, uint16_t y, uint8_t *chunch, uint8_t *tempchunch);
void putcurrent(uint16_t x, uint16_t y, uint16_t *current, uint16_t *tempcurrent);
void putvoltage(uint16_t x, uint16_t y, uint16_t *voltage, uint16_t *voltagetemp);
void putcolumbus(uint16_t x, uint16_t y, uint16_t *columbus, uint16_t *tempcolumbus);
void getaverangevalues(uint16_t *s);
void calculatevalues(uint16_t *s, uint16_t *voltagemeas1, uint16_t *voltagemeas2, uint16_t *voltagemeas3, uint16_t *voltagemeas4,
		             uint16_t *currentmeas1, uint16_t *currentmeas2, uint16_t *currentmeas3, uint16_t *currentmeas4);
void calculatecharge(void);
void setcurrents(uint16_t *current1, uint16_t *current2, uint16_t *current3, uint16_t *current4);
void controlbacklight(void);
void Flash_zapis2(void);
void Flash_odczyt2(void);
void controlports(uint8_t *ch1, uint8_t *ch2, uint8_t *ch3, uint8_t *ch4);
void barcontrol(uint16_t *voltage1, uint16_t *voltage2, uint16_t *voltage3, uint16_t *voltage4, uint16_t *bar1, uint16_t *bar2, uint16_t *bar3, uint16_t *bar4);
void controlwork(void);

//*******************************************************************
//zmienne
volatile uint32_t timerlcd, TIMERUS, touchms, Xtouch, Ytouch, timervalue, timercurrent1, timercurrent2,
		timercurrent3, timercurrent4, timdrg1, timdrg2, timdrg3, timdrg4, timercolumbus, timerval1, timerval2, timervalues,
		timercharge1, timercharge2, timercharge3, timercharge4, timerbacklight, timerbar, timh1, timh2, timh3, timh4;
int chosefont;
unsigned char const wsktemp;
uint8_t flaga1, flaga2=0,
		flagabat1, flagabat2, flagabat3, flagabat4,  chunch1=1, chunch2=1, chunch3=1, chunch4=1, tempchunch1, tempchunch2, tempchunch3,
		tempchunch4, flagacolumbus, flagach1, flagach2, flagach3, flagach4, flagaponownejkalibracji;
uint16_t valuecurrent1, valuecurrent2, valuecurrent3,
valuecurrent4, valuetempcurrent1=200, valuetempcurrent2=200, valuetempcurrent3=200,
valuetempcurrent4=200, voltagebat1=0, voltagebat2=0, voltagebat3=0, voltagebat4=0,
voltagebattemp1=200, voltagebattemp2=200, voltagebattemp3=200, voltagebattemp4=200,
voltagemeas1=121, voltagemeas2=95, voltagemeas3=110, voltagemeas4=87, voltagemeastemp1=1000, voltagemeastemp2=1000,
voltagemeastemp3=1000, voltagemeastemp4=1000, currentmeas1=0, currentmeas2=0, currentmeas3=0, currentmeas4=0,
currentmeastemp1=1000, currentmeastemp2=1000, currentmeastemp3=1000, currentmeastemp4=1000,
columbusmeas1=0, columbusmeas2=0, columbusmeas3=0, columbusmeas4=0, columbusmeastemp1=10000, columbusmeastemp2=10000,
columbusmeastemp3=10000, columbusmeastemp4=10000;
uint32_t *wskkalib;
uint16_t valueADC[8]; //buffor dla pomiarow przetwornika ADC
uint16_t avADC[8];
float aX, bX, dX, aY, bY, dY;   //zmienne potrzebne przy pelnej kalibracji
uint8_t wykalibrowano, globalk=0, globalk2=0;
const uint16_t xcal1=4000, xcal2=100, ycal1=4000, ycal2=100;
uint16_t buftx[4];
uint16_t bufty[4];
uint8_t supvoltagemeas[4];


extern const unsigned char wallpaper1[];
extern const unsigned char wallpaper2[];
extern const uint8_t battery[];


//**********************************************************************
// Definicje dla pamieci flash mikrokontroler

uint32_t EraseCounter = 0x00, Address = 0x00;
uint32_t NbrOfPage = 0x00;
volatile FLASH_Status FLASHStatus=FLASH_COMPLETE;

#define BaseAdresFlash 0x08000000

//ofsety osobno dla funkcji - dla zapamietywania wartosci kalibracyjnych touchapnelu
#define OffsetNazwy2 0x39C00
#define BANK1_WRITE_START_ADDR2  ((uint32_t)0x08039C00)  //Ustawienie 1kb
#define BANK1_WRITE_END_ADDR2    ((uint32_t)0x0803A000)
#define FLASH_PAGE_SIZE         ((uint16_t)0x400)
//************************************************************************

int main(void)
{

	  RCC_Config();
	  NVIC_Config();
	  GPIO_Config();
	  SPI_init();
	  ADC_Config();
	  DMA_Config();
	  TIM_Config();

		if (SysTick_Config_Mod(SysTick_CLKSource_HCLK_Div8, 900)) //przerwanie co 0,1ms
		{
			while(1);		// W razie bledu petla nieskonczona
		}

		LCD_Init(); //inicjalizacja wyswietlacza LCD
		XPT2046_SPI_Init(); //inicjalizacja panelu dotykowego


		 wskkalib=(uint32_t *)(BaseAdresFlash + OffsetNazwy2);  //ustawienie wskaznika na adres komorki ktora jest flaga - czy juz byla doknana kalibracja po pierwszym uruchomienu

		if((*wskkalib!=0x256faba4) || docalibration)
	    {
			if(docalibration) flagaponownejkalibracji=1;
			controlbacklight(); //zalaczenie podswietlenia
			calibrationtouch(); //kalibracja wspolczynnikow ekranu dotykowego
		}

		Flash_odczyt2();
		wykalibrowano=1;  //ustawienie flagi zezwalajacej prace touchpana ze wspolczynnikami korygujacymmi


		//wyswietlenie tla
	    putscaledwallpaper(wallpaper1);
	    //oswiecenie wyswietlacza
	    controlbacklight();
	    //wyswietlenie pierwszej animacji
	    animewallpaper();
	    putmonograph(134, 80, Black1, 0, 96, 160,battery);
	    putmonograph(249, 80, Black1, 0, 96, 160,battery);
	    putmonograph(19, 80,  Black1, 0, 96, 160,battery);
	    putmonograph(364, 80, Black1, 0, 96, 160,battery);

	    //Uruchomienie konwersji dla przetwornika ADC
	    ADC_SoftwareStartConvCmd(ADC1, ENABLE);

//***************************************************************************************
//Glowna petla programu
		while(1)
		{
			//Animacja dwoch paskow
			 animwallpaper2();

			 //Narysowanie na wyswietlaczu aktualnych stanow poziomow naladowania baterii
			 if(!timervalue || flagabat1 || flagabat2 || flagabat3 || flagabat4)
			 {

			if(!timercurrent1) barbattery(&voltagebat1, &voltagebattemp1, &flagabat1, 19, 80);
			if(!timercurrent2) barbattery(&voltagebat2, &voltagebattemp2, &flagabat2, 134, 80);
			if(!timercurrent3) barbattery(&voltagebat3, &voltagebattemp3, &flagabat3, 249, 80);
			if(!timercurrent4) barbattery(&voltagebat4, &voltagebattemp4, &flagabat4, 364, 80);

			 timervalue=4000; //co 400ms aktualizacja stanu naladowania baterii
			 }

			if(timercurrent1)  setcurrent(&valuecurrent1, &valuetempcurrent1, 19, 80);
			if(timercurrent2)  setcurrent(&valuecurrent2, &valuetempcurrent2, 134, 80);
			if(timercurrent3)  setcurrent(&valuecurrent3, &valuetempcurrent3, 249, 80);
			if(timercurrent4)  setcurrent(&valuecurrent4, &valuetempcurrent4, 364, 80);

		    touchcontrol();  //obsluga panelu dotykowego

		    //wyswietlenie przyciskow
		    putbuttons(244, 275, &chunch3, &tempchunch3);
		    putbuttons(129, 275, &chunch2, &tempchunch2);
		    putbuttons(14,  275, &chunch1, &tempchunch1);
		    putbuttons(359, 275, &chunch4, &tempchunch4);

		    //wyswietlenie wartosci napiec
		    if(!timerval1)
		    {
		    putvoltage(34, 40, &voltagemeas1, &voltagemeastemp1);
		    putvoltage(149, 40, &voltagemeas2, &voltagemeastemp2);
		    putvoltage(264, 40, &voltagemeas3, &voltagemeastemp3);
		    putvoltage(379, 40, &voltagemeas4, &voltagemeastemp4);
		    timerval1=1100;
		    }

		    //wyswietlenie wartosci pradow
		    if(!timerval2)
		    {
		    putcurrent(34, 60, &currentmeas1,  &currentmeastemp1);
		    putcurrent(149, 60, &currentmeas2, &currentmeastemp2);
		    putcurrent(264, 60, &currentmeas3, &currentmeastemp3);
		    putcurrent(379, 60, &currentmeas4, &currentmeastemp4);
		    timerval2=1200;
		    }

		    //wyswietlenie wartosci ladunku ktory przeplynal od momentu zmiany ladowanie/rozladowanie
		    if((!timercolumbus) || flagacolumbus)
		    {
		     putcolumbus(21, 245, &columbusmeas1, &columbusmeastemp1);
		     putcolumbus(136, 245, &columbusmeas2, &columbusmeastemp2);
		     putcolumbus(251, 245, &columbusmeas3, &columbusmeastemp3);
		     putcolumbus(366, 245, &columbusmeas4, &columbusmeastemp4);
		     timercolumbus=1800;
		     flagacolumbus=0;
		    }

		    //funkcja liczaca srednie wartosci z przetwornica ADC
		    getaverangevalues(avADC);

		    //funkcja przeliczajaca wartosci na napiecie oraz prad
		    calculatevalues(avADC, &voltagemeas1, &voltagemeas2, &voltagemeas3, &voltagemeas4, &currentmeas1, &currentmeas2, &currentmeas3, &currentmeas4);

		    //procedura przeliczajaca wartosc ladunku ktory przeplynal
		    calculatecharge();

		    //funkcja ustawiajaca aktualna wartosc ustawionego pradu dla kanalow
		    setcurrents(&valuecurrent1, &valuecurrent2, &valuecurrent3, &valuecurrent4);

		    //kontrola portow sterujacych
		    controlports(&chunch1, &chunch2, &chunch3, &chunch4);

		    //funkcja kontrolujaca pasek stanu
		    barcontrol(&voltagemeas1, &voltagemeas2, &voltagemeas3, &voltagemeas4, &voltagebat1, &voltagebat2, &voltagebat3, &voltagebat4);

		    //funkcja kontrolujaca logike ladowarki
		    controlwork();
		}

}


void GPIO_Config(void)
{

	 //konfigurowanie portow dla wyswietlacza SPI
	  GPIO_InitTypeDef  GPIO_InitStructure;

	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_Init(GPIOA, &GPIO_InitStructure);

	    GPIOA->BSRR = GPIO_Pin_4;

	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIOA->BSRR = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;

  //*********************************************************************************
  //Dla kontroli sterowania wyswietlacz PIN9, PIN10, PIN11, PIN12
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  GPIOA ->BRR = GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12;

  //Sterowanie podswietleniem
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  GPIOB->BSRR = GPIO_Pin_1;


  //Konfiguracja wejscia przetwornika jako wejscia analogowego - ADC
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 |GPIO_Pin_2 | GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 |GPIO_Pin_2 | GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  //Wyjscie dla ukladow PWM - DAC
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_1;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP; //wyjscie alternatywne o wysokiej impedancji
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  //Przycisk kalibracji
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  GPIOB ->BSRR = GPIO_Pin_0;

  //obsluga portow sterujacych ladowanie oraz rozladowaniem
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_6 | GPIO_Pin_7;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

   GPIOC->BRR=GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_6 | GPIO_Pin_7;

   //Obsluga diod informacyjnych
   GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
   GPIO_Init(GPIOC, &GPIO_InitStructure);

   GPIOC->BRR= GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12;

   //Obsluga diod informacyjnych
      GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2 ;
      GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
      GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
      GPIO_Init(GPIOD, &GPIO_InitStructure);

      GPIOD->BRR=  GPIO_Pin_2;
}


void NVIC_Config(void)
{

	NVIC_InitTypeDef    NVIC_InitStructure;

	NVIC_InitStructure.NVIC_IRQChannel =SysTick_IRQn ; //tutaj trzeba zwrocic uwage na definicje w pliu stm32f10x.h
	//******************************************************************************************************************
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);
}


void SPI_init(void)
{

   SPI_InitTypeDef  SPI_InitStructure;    //definicja obiektu stuktury

   SPI_I2S_DeInit(SPI1);

   SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;    //ustawienie trybu 2 lini - MOSI i MISO
   SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;                     //ustawienie dlugosci danych przesylanych
   SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;                            //zegar w stanie niskim
   SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;                          //reakcja na pierwsze zbocze zegarowe
   SPI_InitStructure.SPI_NSS = SPI_NSS_Hard;							 //ustawienie obslugi wyjscia CS programowo
   SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2;    //preskaler dla zegara taktujacego SPI - 48Mhz/2=24Mhz
   SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;   				 //ustawienie najbardziej znacxacego bitu jako wysylany pierwszy
   SPI_InitStructure.SPI_CRCPolynomial = 0;                              //kontrola przeslanych bitow
   SPI_InitStructure.SPI_Mode = SPI_Mode_Master;   						 //tryb master
   SPI_Init(SPI1, &SPI_InitStructure);									 //wpisanie parametrow do struktury SPI1
   SPI_SSOutputCmd(SPI1, ENABLE);
   SPI_Cmd(SPI1, ENABLE);												 //funkcja zalaczajaca interfejs szerego SPI
}


void RCC_Config(void)
{
	  RCC_DeInit();                                          //Reset ustawien RCC
	  RCC_HSEConfig(RCC_HSE_ON);                             //Wlaczenie HSE



		RCC_HCLKConfig(RCC_SYSCLK_Div1);                     //ustaw HCLK=SYSCLK
		RCC_PCLK2Config(RCC_HCLK_Div1);                      //ustaw PCLK2=HCLK
		RCC_PCLK1Config(RCC_HCLK_Div1);                      //ustaw PCLK1=HCLK/2
	    RCC_PLLConfig(RCC_PLLSource_PREDIV1, RCC_PLLMul_9); //ustaw PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz

	    RCC_PLLCmd(ENABLE);                                  //wlacz PLL
	    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET){}  //odczekaj na poprawne uruchomienie PLL
	    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           //ustaw PLL jako zrodlo sygnalu zegarowego
	    while(RCC_GetSYSCLKSource() != 0x08){}                //odczekaj az PLL bedzie sygnalem zegarowym systemu

	  //kod zwiazany z konfiguracja sygnalow zegarowych potrzebnych w programie peryferiow

	        RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE); //wlaczenie wyjsc alternatywnych


			RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
			RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
			RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
			RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);


			RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);
			RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE); //zalaczenie taktowania SPI1 dla wyswietlacza

	        //Zalaczenie szyn odpowiedzialnych za pomiary wartosci
		     RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE); //wlacz taktowanie ADC1
		    //Podzia czestotliwosci dla ADC
		     RCC_ADCCLKConfig(RCC_PCLK2_Div8);

		     RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);   //wlacz taktowanie DMA

		     RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE); //wlacz taktowanie TIM4
		     RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE); //wlacz taktowanie TIM3

	        SystemCoreClockUpdate ();   //aktualizacja zmiennych potrzebnych w bibliotekach do przeliczenia wspolczynnikow
}


void DMA_Config(void)
{
  //konfigurowanie DMA

  #define ADC1_DR_Address 0x4001244C;                                             //adres rejestru ADC1->DR

  DMA_InitTypeDef DMA_InitStructure;

  DMA_DeInit(DMA1_Channel1);                                                      //Usun ewentualna poprzednia konfiguracje DMA

  DMA_InitStructure.DMA_PeripheralBaseAddr = (unsigned long int)ADC1_DR_Address;  //Adres docelowy transferu
  DMA_InitStructure.DMA_MemoryBaseAddr = (unsigned long int)valueADC;            //Adres poczatku bloku do przeslania
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;                              //Kierunek transferu
  DMA_InitStructure.DMA_BufferSize = 8;                                           //Liczba elementow do przeslania (dlugosc bufora)
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;                //Wylaczenie automatycznego zwiekszania adresu po stronie ADC
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;                         //Wylaczenie automatycznego zwiekszania adresu po stronie pamieci (bufora)
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;     //Rozmiar pojedynczych przesylanych danych po stronie ADC (HalfWord = 16bit)
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;             //Rozmiar pojedynczych przesylanych danych po stronie pamieci (bufora)
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;                                 //Tryb dzialania kontrolera DMA - powtarzanie cykliczne
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;                             //Priorytet DMA - wysoki
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;                                    //Wylaczenie obslugi transferu z pamieci do pamieci
  DMA_Init(DMA1_Channel1, &DMA_InitStructure);                                    //Zapis konfiguracji

  //Wlacz DMA, kanal 1
  DMA_Cmd(DMA1_Channel1, ENABLE);
}


//Konfiguracja ADC
void ADC_Config(void)
{
  //konfigurowanie przetwornika AC
  ADC_InitTypeDef ADC_InitStructure;

	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;                  //Jeden przetwornik, praca niezalezna
	ADC_InitStructure.ADC_ScanConvMode = ENABLE;                        //Pomiar jednego kanalu, brak skanowania kanalow
    ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;                  //Pomiar w trybie ciaglym
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;	//Brak wyzwalania zewnetrznego
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;            	//Wyrownanie danych do prawej - 12 mlodszych bitow znaczacych
	ADC_InitStructure.ADC_NbrOfChannel = 8; 	                          //Liczba uzywanych kanalow =1
	ADC_Init(ADC1, &ADC_InitStructure);                                 //Incjalizacja przetwornika

    ADC_RegularChannelConfig(ADC1, ADC_Channel_0, 1, ADC_SampleTime_239Cycles5);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 2, ADC_SampleTime_239Cycles5);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_2, 3, ADC_SampleTime_239Cycles5);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_3, 4, ADC_SampleTime_239Cycles5);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_10, 5, ADC_SampleTime_239Cycles5);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_11, 6, ADC_SampleTime_239Cycles5);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_12, 7, ADC_SampleTime_239Cycles5);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_13, 8, ADC_SampleTime_239Cycles5);
                                                                      //Kanal 1 - GPIOA1
                                                                      //Grupa podstawowa, czas probkowania 41,5+12,5 cykli (calkowity czas przetwarania)
   ADC_DMACmd(ADC1,ENABLE);                                            //Wlaczenie DMA dla ADC
   ADC_Cmd(ADC1, ENABLE);	                                            //Wlacz ADC1

   ADC_ResetCalibration(ADC1);	                                        //Reset rejestrow kalibracyjnych ADC1
   while(ADC_GetResetCalibrationStatus(ADC1));	                        //Odczekanie na wykonanie resetu
   ADC_StartCalibration(ADC1);	                                        //Kalibracja ADC1
   while(ADC_GetCalibrationStatus(ADC1));    	                        //Odczekanie na zakonczenie kalibracji ADC1
}


void TIM_Config(void)
{
  //konfigurowanie licznikow

	//***************************************************************************************************
	//Poczatek konfiguracja timer3 odpowiedzialnego za jasnosc wyswietlacza
	//***************************************************************************************************

  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;

  // Konfiguracja TIM3
  // Ustawienia ukladu podstawy czasu
  TIM_TimeBaseStructure.TIM_Prescaler = 0;         //bez prescelera (prescaler=1)
  TIM_TimeBaseStructure.TIM_Period = 1000ul;      //czestotliwosc PWM =5,86kHz  poniewaz 72Mhz/8000=9kHz
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);
  TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);


  // Konfiguracja kanalu 1, 2, 3, 4 - uzywamy kanalu 3 poniewaz jego wyjscie jest na GPIOB0
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1; //tryb pwm
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 0ul;          //poczatkowe wypelnienie maksymalne (TIMpulse/Timperiod)*100%=100%
  TIM_OCInitStructure.TIM_OCPolarity=TIM_OCPolarity_High;
  TIM_OC3Init(TIM4, &TIM_OCInitStructure);
  TIM_OC4Init(TIM4, &TIM_OCInitStructure);
  TIM_OC2Init(TIM4, &TIM_OCInitStructure);
  TIM_OC1Init(TIM4, &TIM_OCInitStructure);

  TIM_OC4Init(TIM3, &TIM_OCInitStructure);

  TIM_OC4PreloadConfig(TIM4, TIM_OCPreload_Enable);//wlaczenie buforowania
  TIM_OC3PreloadConfig(TIM4, TIM_OCPreload_Enable);//wlaczenie buforowania
  TIM_OC2PreloadConfig(TIM4, TIM_OCPreload_Enable);//wlaczenie buforowania
  TIM_OC1PreloadConfig(TIM4, TIM_OCPreload_Enable);//wlaczenie buforowania

  TIM_OC4PreloadConfig(TIM3, TIM_OCPreload_Enable);//wlaczenie buforowania

  TIM_ARRPreloadConfig(TIM4, ENABLE);//wlaczenie buforowania
  TIM_ARRPreloadConfig(TIM3, ENABLE);//wlaczenie buforowania
  // Wlaczenie timera
  TIM_Cmd(TIM4, ENABLE);
  TIM_Cmd(TIM3, ENABLE);

}

//---------------------------------------------------------------------------------------------------------------
//---------------------- Funkcje obslugujace pamiec flash uC-----------------------------------------------------

//Funkcja odczytujaca wspolczynniki dla kalibracji touchpanelu
void Flash_odczyt2(void)
{
	uint32_t *p;
	uint32_t  t;
	unsigned int wspolczynnikkalibracji[6];

	 union
	    {
	     unsigned int i;
	     float f;
	     }var1,var2, var3, var4, var5, var6;


	p=(uint32_t *)(BaseAdresFlash+OffsetNazwy2+4);

	for(t=0; t<=5; t++)
	{
	 wspolczynnikkalibracji[t]=*p; //przepisanie wartosci z fla
	  p++;
	}

	var1.i = wspolczynnikkalibracji[0];
	var2.i = wspolczynnikkalibracji[1];
	var3.i = wspolczynnikkalibracji[2];
	var4.i = wspolczynnikkalibracji[3];
	var5.i = wspolczynnikkalibracji[4];
	var6.i = wspolczynnikkalibracji[5];

	aX=var1.f;
	bX=var2.f;
	dX=var3.f;
	aY=var4.f;
	bY=var5.f;
	dY=var6.f;

}

// Wersje drugie dla wspolczynnikow kalibracji panelu oraz kalibracji pomiaru
void Flash_zapis2(void)
{
	 uint32_t   i;
     volatile uint32_t  j;
     unsigned int wspolczynnikkalibracji[6];
     uint32_t *k;

     union
     {
     unsigned int i;
     float f;
     }var1,var2, var3, var4, var5, var6;

     //zrzutowanie zmiennych przecinkowych na unie aby mozna je bylo potem wpisac do pamieci flash jako unsigned int
     var1.f=aX;
     var2.f=bX;
     var3.f=dX;
     var4.f=aY;
     var5.f=bY;
     var6.f=dY;

     wspolczynnikkalibracji[0]=var1.i;
     wspolczynnikkalibracji[1]=var2.i;
     wspolczynnikkalibracji[2]=var3.i;
     wspolczynnikkalibracji[3]=var4.i;
     wspolczynnikkalibracji[4]=var5.i;
     wspolczynnikkalibracji[5]=var6.i;



 	k=(uint32_t *)(BaseAdresFlash + OffsetNazwy2);  //ustawienie wskaznika na adres komorki ktora jest flaga czy juz byl dokonany zapis i pierwsze uruchonmienie

 	if(*k!=0x256faba4  || flagaponownejkalibracji) //sprawdzenie po zalaczeniu czy pamiec juz byla uzywana i nie jest to pierwszy raz po zapisie
 	{

	//Odblokowanie flash

	FLASH_Unlock();

	NbrOfPage = (BANK1_WRITE_END_ADDR2 - BANK1_WRITE_START_ADDR2) / FLASH_PAGE_SIZE;

	FLASH_ClearFlag(FLASH_FLAG_EOP|FLASH_FLAG_PGERR|FLASH_FLAG_WRPRTERR);




  // Wykasowanie pamieci
  for(EraseCounter = 0; (EraseCounter < NbrOfPage) && (FLASHStatus == FLASH_COMPLETE); EraseCounter++)
  {
    FLASHStatus = FLASH_ErasePage(BANK1_WRITE_START_ADDR2 + (FLASH_PAGE_SIZE * EraseCounter));
  }


		while(FLASH_COMPLETE!=FLASH_ProgramWord(BaseAdresFlash + OffsetNazwy2, 0x256faba4))   //zmienna blokujaca nadpisanie przy uruchomieniu
			{
			}
		j=(BaseAdresFlash+OffsetNazwy2+4);


		//W tym miejscu nastepuje wpisanie zmiennych kalibracyjnych touchpanelu

		for(i=0; i<=5; i++)
		{
			 while(FLASH_COMPLETE!=FLASH_ProgramWord(j, wspolczynnikkalibracji[i]))
			 {
			 }
			 j=j+4;

	    }

	FLASH_Lock();  //zablokowanie dostepu do pamieci flash
 	}
}

void calibrationtouch(void)
{

	uint16_t touchval[3][3] = {{1,1,1}, {1,1,1}, {1,1,1}};   //Talbica wartosci z przetwornika przy czym
	//pierwszy element x, drugi y a trzeci 1 - poniewaz tak wyglada macierz
	int32_t transtouch[3][3];                                     //macierz trans dla macierzy touchval
	uint32_t dettouch; 										  //wyznacznik macierzy touchval
	//wspolrzende punktow ktore beda rysowane
	uint16_t wsppx[3]={15, 240, 470};
	uint16_t wsppy[3]={15, 300, 30};
	float invtouch[3][3];                       // inwersja  macierzy A
	float temporarytouch;

	uint8_t ip, jp;

	LCD_Clear(White); //Wyczyszczenie wywietlacza biaym tem
    chosefont=5;
	tft_mputs( 100, 130,"Please touch and hold  ", Red2, White,wallpaper2, 0);
	tft_mputs( 120, 150,"  visible squares    ", Red2, White,wallpaper2, 0);

	TIMERUS=20000;  //odczekanie 2s
	while(TIMERUS);

												  //Pierwszy punkt
	full_rect( 14, 14, 16, 16, Red1);            //punkt x=15 y=15



	TIMERUS=20000;  //odczekanie 2s
	while(TIMERUS)
	{
		gettouchvalue();   //pobieraie wartosci z kontrolera oraz usrednianie
	}


	touchval[0][0] = Xtouch;
	touchval[0][1] = Ytouch;

	full_rect( 14, 14, 16, 16, White);			  //Drugi punkt
	full_rect( 239, 299, 241, 301, Red1);        //punkt x=160, y=225


	TIMERUS=20000; //odczekanie 2s
	while(TIMERUS)
	{
		gettouchvalue();   //pobieraie wartosci z kontrolera oraz usrednianie
	}


	touchval[1][0] = Xtouch;
	touchval[1][1] = Ytouch;

	full_rect( 239, 299, 241, 301, White);      //Trzeci punkt
	full_rect( 469, 29, 471, 31, Red1);        // punkt x=305 y=120



	TIMERUS=20000; //odczekanie 2s
	while(TIMERUS)
	{
		gettouchvalue();   //pobieraie wartosci z kontrolera oraz usrednianie
	}


	touchval[2][0] = Xtouch;
	touchval[2][1] = Ytouch;

	LCD_Clear(White);    //wyczyszczenie ekranu

	//obliczanie wyznacznika macierzy pobranych polozen z rownania
	dettouch = ((touchval[0][0])*(((touchval[1][1])*(touchval[2][2]))-((touchval[1][2])*(touchval[2][1])))) +
	((touchval[0][1])*(((touchval[1][2])*(touchval[2][0])) - ((touchval[2][2])*(touchval[1][0])))) +
	((touchval[0][2])*(((touchval[1][0])*(touchval[2][1])) - ((touchval[1][1])*(touchval[2][0]))));

	//obliczanie elementow macierzy odwroconej za pomoca wzorow - na minior (elementy jeszcze nie podzielone przez wyznacznik)
	transtouch[0][0] = (touchval[1][1] * touchval[2][2]) - (touchval[1][2] * touchval[2][1]);
	transtouch[1][0] = (touchval[1][2] * touchval[2][0]) - (touchval[1][0] * touchval[2][2]);
	transtouch[2][0] = (touchval[1][0] * touchval[2][1]) - (touchval[1][1] * touchval[2][0]);
	transtouch[0][1] = (touchval[2][1] * touchval[0][2]) - (touchval[0][1] * touchval[2][2]);
	transtouch[1][1] = (touchval[0][0] * touchval[2][2]) - (touchval[2][0] * touchval[0][2]);
	transtouch[2][1] = (touchval[2][0] * touchval[0][1]) - (touchval[0][0] * touchval[2][1]);
	transtouch[0][2] = (touchval[0][1] * touchval[1][2]) - (touchval[1][1] * touchval[0][2]);
	transtouch[1][2] = (touchval[0][2] * touchval[1][0]) - (touchval[0][0] * touchval[1][2]);
	transtouch[2][2] = (touchval[0][0] * touchval[1][1]) - (touchval[0][1] * touchval[1][0]);

	//Obliczenie odwrotnosci
	temporarytouch = 1/((float)dettouch);

	//obliczenie wszystkich elementow odwrotnej macierzy
	for(ip=0;ip<3;ip++)
	{
	for(jp=0;jp<3;jp++)
	{
	invtouch[ip][jp] = temporarytouch * ((float)transtouch[ip][jp]);
	}
	}

	aX = invtouch[0][0]*(float)wsppx[0] + invtouch[0][1]*(float)wsppx[1] +
	invtouch[0][2]*(float)wsppx[2];
	bX = invtouch[1][0]*(float)wsppx[0] + invtouch[1][1]*(float)wsppx[1] +
	invtouch[1][2]*(float)wsppx[2];
	dX = invtouch[2][0]*(float)wsppx[0] + invtouch[2][1]*(float)wsppx[1] +
	invtouch[2][2]*(float)wsppx[2];

	aY = invtouch[0][0]*(float)wsppy[0] + invtouch[0][1]*(float)wsppy[1] +
		 invtouch[0][2]*(float)wsppy[2];
	bY = invtouch[1][0]*(float)wsppy[0] + invtouch[1][1]*(float)wsppy[1] +
		 invtouch[1][2]*(float)wsppy[2];
	dY = invtouch[2][0]*(float)wsppy[0] + invtouch[2][1]*(float)wsppy[1] +
		 invtouch[2][2]*(float)wsppy[2];

	//w tym miejscu nastepuje zapis wspolczynnikow kalibracji do pamieci flash
	//-------------------------------------------------------------------
	Flash_zapis2();
	//-------------------------------------------------------------------

}


//funkcja zwracajaca wartosci wspolrzednych po dotknieciu ekranu
 uint8_t gettouchvalue(void)
 {
 	float temp1, temp2;
 	uint8_t k;
 	uint8_t temp=0;

 	uint32_t bigx=0, bigy=0;

 	if(!touchms)
 	{
 		touchms=50; //5ms pomiedzy pomiarami


 		bufty[globalk]=ADS_Read_AD(CMD_RDX);
 		buftx[globalk]=ADS_Read_AD(CMD_RDY);


 		//Wyelyminowanie dodatkowych zaklocen (fluktuacji )
 		if(buftx[globalk]>100  && buftx[globalk]<4000  && bufty[globalk]>100  && bufty[globalk]<4000)
 		globalk++;
 		else globalk=0;
 	}

 	 if(globalk==4)
 			 		{
 			 			k=globalk;
 			 			//wykasowanie poprzedniego wyniku
 			 			globalk=0;

 			 			for(k=0;k<4; k++)
 			 			{

 			 				bigx+=buftx[k];
 			 				bigy+=bufty[k];
 			 			}

 			 			Xtouch=bigx/k;
 			 			Ytouch=bigy/k;


 		if(wykalibrowano)
 		{

 			if(Ytouch>ycal2 && Ytouch<ycal1 && Xtouch>xcal2 && Xtouch<xcal1) //jezeli odczytana wartosc miesci sie w granicach zmiennych granicznych
 			{

 			temp1 = aX*(float)Xtouch + bX*(float)Ytouch + dX;
 			temp2 = aY*(float)Xtouch + bY*(float)Ytouch + dY;

 			Xtouch=temp1;
 			Ytouch=temp2;
 			temp=1;


 		    }
 			else
 				{
 					Xtouch=0;
 					Ytouch=0;
 					temp=0;
 				}

 			return temp;
 		 }

 			 		}
 	 return temp;

 }



 //procedura poczatkowej animacji
 void animewallpaper(void)
 {
	 for(uint8_t i=0; i<152; i++)
	 {

		 putrectfromwallpaperscaled(7, 159-i, 473, 160-i, wallpaper2, 0);
		 putrectfromwallpaperscaled(7, 160+i, 473, 161+i, wallpaper2, 0);
		 //odczekanie 20ms
		 TIMERUS=200;
		 while(TIMERUS);
	 }
 }

 //animowane belki u gory oraz u dolu
 void animwallpaper2(void)
 {
	 static uint8_t var=0, flagflag=1;


	 if(!TIMERUS)
	 {
		 if(flagflag) var++; else var--;
		 if(var>7) flagflag=0; else if(var==0) flagflag=1;

		 if(flagflag)
		 {
		 transparentfromwallpaper(7, 6+var, 473, 7+var, wallpaper1, 20, 5, 2, 1);
		 transparentfromwallpaper(7, 312-var, 473, 313-var, wallpaper1, 20, 5, 2, 1);
		 }
		 else
		 {
			 transparentfromwallpaper(7, 6+var, 473, 7+var, wallpaper2, 0, 0, 0, 1);
			 transparentfromwallpaper(7, 312-var, 473, 313-var, wallpaper2, 0, 0, 0, 1);
		 }

		 TIMERUS=1000;
	 }
 }

 //wyswietlanie na wyswietlaczu aktualnych wartosci poziomow naladowania baterii
 void barbattery(uint16_t *value, uint16_t *valuetemp, uint8_t *flagapomoc, uint16_t x, uint16_t y)
 {
	 uint16_t temphigh;

	 if((*value!=*valuetemp) || (*flagapomoc)) //jezeli wartosc sie zmienila naladowania bateri
	 {
		 *valuetemp=*value;
		 *flagapomoc=0;

		 temphigh=126*(*value);
		 temphigh=temphigh/100;
		 if(temphigh==0) temphigh=1; else if(temphigh==126) temphigh=125;

		 transparentfromwallpaper(12+x, y+23, x+83, y+149-temphigh, wallpaper2, 0, 0, 0, 1);
		 transparentfromwallpaper(12+x, y+149-temphigh, x+83, y+148, wallpaper1, 15, 0, 20, 0);
	 }
}

//wyswietla na wyswietlaczu aktualnie wybrana wartosc pradu
 void setcurrent(uint16_t *value, uint16_t *valuetemp, uint16_t x, uint16_t y)
 {
	 uint16_t temphigh;

	 if(*value!=*valuetemp) //jezeli wartosc sie zmienila naladowania bateri
	 {
		 *valuetemp=*value;

		 temphigh=126*(*value);
		 temphigh=temphigh/100;
		 if(temphigh==0) temphigh=1; else if(temphigh==126) temphigh=125;

		 transparentfromwallpaper(12+x, y+23, x+83, y+149-temphigh, wallpaper2, 0, 0, 0, 1);
		 transparentfromwallpaper(12+x, y+149-temphigh, x+83, y+148, wallpaper1, 30, 30, 0, 0);
	 }
}

//Procedura kontrolujaca panel dotykowy
void touchcontrol(void)
{

	uint16_t temptempvalue;

		//Sprawdzenie czy nastapilo nacisniecie przycisku
		if(gettouchvalue())
	    {
		     if(Xtouch>14 && Xtouch<120 && Ytouch>70 && Ytouch<260)
		     {
		    	 if((Ytouch>80) && (Ytouch<240))
		    		 {
		    		 temptempvalue=240-Ytouch;
		    		 }
		    	 else if(Ytouch>=240) temptempvalue=0; else temptempvalue=160;

		    	 temptempvalue=temptempvalue*100;
		    	 temptempvalue=temptempvalue/160;

		    	 valuecurrent1=temptempvalue;
		    	 valuetempcurrent1=200;
		    	 timercurrent1=30000;
		    	 flagabat1=1;

		     }
		     if(Xtouch>129 && Xtouch<235 && Ytouch>70 && Ytouch<260)
		     {
		    	 if((Ytouch>80) && (Ytouch<240))
		    			    		 {
		    			    		 temptempvalue=240-Ytouch;
		    			    		 }
		    			    	 else if(Ytouch>=240) temptempvalue=0; else temptempvalue=160;

		    			    	 temptempvalue=temptempvalue*100;
		    			    	 temptempvalue=temptempvalue/160;

		    			    	 valuecurrent2=temptempvalue;
		    			    	 valuetempcurrent2=200;
		    			    	 timercurrent2=30000;
		    			    	 flagabat2=1;

		     }
		     if(Xtouch>244 && Xtouch<349 && Ytouch>70 && Ytouch<260)
		     {
		    	 if((Ytouch>80) && (Ytouch<240))
		    			    		 {
		    			    		 temptempvalue=240-Ytouch;
		    			    		 }
		    			    	 else if(Ytouch>=240) temptempvalue=0; else temptempvalue=160;

		    			    	 temptempvalue=temptempvalue*100;
		    			    	 temptempvalue=temptempvalue/160;

		    			    	 valuecurrent3=temptempvalue;
		    			    	 valuetempcurrent3=200;
		    			    	 timercurrent3=30000;
		    			    	 flagabat3=1;

		     }
		     if(Xtouch>359 && Xtouch<465 && Ytouch>70 && Ytouch<260)
		     {
		    	 if((Ytouch>80) && (Ytouch<240))
		    			    		 {
		    			    		 temptempvalue=240-Ytouch;
		    			    		 }
		    			    	 else if(Ytouch>=240) temptempvalue=0; else temptempvalue=160;

		    			    	 temptempvalue=temptempvalue*100;
		    			    	 temptempvalue=temptempvalue/160;

		    			    	 valuecurrent4=temptempvalue;
		    			    	 valuetempcurrent4=200;
		     		     		 timercurrent4=30000;
		     		     		 flagabat4=1;
		     }
		     //obsluga zalacz wylacz
		     if(Xtouch>14 && Xtouch<129 && Ytouch>270 && Ytouch<300)
		     {
		    	 if(!timdrg1)
		    		 {
		    		 if(chunch1) chunch1=0; else chunch1=1;
		    		 timdrg1=1000;
		    		 columbusmeas1=0;
		    		 flagacolumbus=1;
		    		 flagach1=1;
		    		 }
		     }
		     if(Xtouch>129 && Xtouch<244 && Ytouch>270 && Ytouch<300)
		   	 {
		    	 if(!timdrg2)
		    		 {
		    		 if(chunch2) chunch2=0; else chunch2=1;
		    		 timdrg2=1000;
		    		 columbusmeas2=0;
		    		 flagacolumbus=1;
		    		 flagach2=1;
		    		 }
		     }
		     if(Xtouch>244 && Xtouch<359 && Ytouch>270 && Ytouch<300)
		     {
		    	 if(!timdrg3)
		    		 {
		    		 if(chunch3) chunch3=0; else chunch3=1;
		    		 timdrg3=1000;
		    		 columbusmeas3=0;
		    		 flagacolumbus=1;
		    		 flagach3=1;
		    		 }
		   	 }
		     if(Xtouch>359 && Xtouch<464 && Ytouch>270 && Ytouch<300)
		     {
		    	 if(!timdrg4)
		    		 {
		    		 if(chunch4) chunch4=0; else chunch4=1;
		    		 timdrg4=1000;
		    		 columbusmeas4=0;
		    		 flagacolumbus=1;
		    		 flagach4=1;
		    		 }
		     }


	     }
}

//Funkcja rysujaca na wyswietlaczu aktualny stan przycisku
void putbuttons(uint16_t x, uint16_t y, uint8_t *chunch, uint8_t *tempchunch)
{
	if(*chunch!=*tempchunch)
	{
		*tempchunch=*chunch;
		chosefont=5; //wybranie czcionki 5

		if(*chunch)
		{
		 tft_round_rect(x, y, 105, 24, 5, Black1, Black1);
		 tft_mputs( x+14, y+2, "Charge", White, 0, wallpaper2, 0);
		}
		else
		{
		 tft_round_rect(x, y, 105, 24, 5, White, White);
		 tft_mputs( x+2, y+2, "Uncharge", Black1, 0, wallpaper2, 0);
		}
	}

}

//Funkcja wypisujaca wartosc napiecia  - format 1.53 -> 153
void putvoltage(uint16_t x, uint16_t y, uint16_t *voltage, uint16_t *voltagetemp)
{
	char buffor[6];


	if(*voltage!=*voltagetemp)
	{
		chosefont=5;

		*voltagetemp=*voltage;

		if(*voltage>999) *voltage=999;

		buffor[0]=(*voltage/100);
		buffor[1]=',';
		buffor[2]=*voltage-(100*buffor[0]);
		buffor[2]=buffor[2]/10;
		buffor[3]=(*voltage)%10;
		buffor[4]='V';
		buffor[5]=0x00;

		buffor[0]+=0x30;
		buffor[2]+=0x30;
		buffor[3]+=0x30;

		 tft_mputs( x, y, buffor, Black1, 0, wallpaper2, 2);
	}
}

//funkcja wypisujaca wartosc pradu - format 235mA -> 235
void putcurrent(uint16_t x, uint16_t y, uint16_t *current, uint16_t *tempcurrent)
{
	char buffor[6];


	if(*current!=*tempcurrent)
	{
		chosefont=5;

		*tempcurrent=*current;

		if(*current>999) *current=999;

		buffor[0]=(*current/100);
		buffor[1]=*current-(100*buffor[0]);
		buffor[1]=buffor[1]/10;
		buffor[2]=(*current)%10;
		buffor[3]='m';
		buffor[4]='A';
		buffor[5]=0x00;

		buffor[0]+=0x30;
		buffor[1]+=0x30;
		buffor[2]+=0x30;

		if(buffor[0]==0x30)
		{
			buffor[0]=247;
			if(buffor[1]==0x30) buffor[1]=247;
		}

		tft_mputs( x, y, buffor, Black1, 0, wallpaper2, 2);
	}
}

//funkcja wypisujaca wartosc pojemnosci - format 235mAh -> 235
void putcolumbus(uint16_t x, uint16_t y, uint16_t *columbus, uint16_t *tempcolumbus)
{
	char buffor[8];
	uint16_t temptemp;


	if(*columbus!=*tempcolumbus)
	{
		chosefont=5;

		*tempcolumbus=*columbus;


		if(*columbus>9999) *columbus=9999;

		buffor[0]=(*columbus/1000);
		temptemp=*columbus-(1000*buffor[0]);
		buffor[1]=temptemp/100;
		buffor[2]=temptemp-buffor[1]*100;
		buffor[2]=buffor[2]/10;
		buffor[3]=(*columbus)%10;
		buffor[4]='m';
		buffor[5]='A';
		buffor[6]='h';
		buffor[7]=0x00;

		buffor[0]+=0x30;
		buffor[1]+=0x30;
		buffor[2]+=0x30;
		buffor[3]+=0x30;

		if(buffor[0]==0x30)
		{
			buffor[0]=247;
			if(buffor[1]==0x30)
				{
					buffor[1]=247;
					if(buffor[2]==0x30) buffor[2]=247;
				}
		}

		tft_mputs( x, y, buffor, Black1, 0, wallpaper2, 2);
	}
}

//procedura pobierajaca dane z przetornikow ADC oraz obliczajaca srednia
void getaverangevalues(uint16_t *s)
{
	static uint16_t bufforval[8][5]= { {0,0,0,0,0}, {0,0,0,0,0}, {0,0,0,0,0}, {0,0,0,0,0},
			                          {0,0,0,0,0}, {0,0,0,0,0}, {0,0,0,0,0}, {0,0,0,0,0} };
	static uint8_t in=0;
	static uint8_t kn=0;
	uint16_t *ka;
	uint32_t temptemp;

	ka=s;

	//jezeli nastapil odpowiedni czas to nalezy sprawdzic czas
	if(!timervalues)
	{
		kn++;
		if(kn>7)
		{
			kn=0;
			in++;
			if(in>4) in=0;
		}

		bufforval[kn][in]=valueADC[kn];

		for(uint8_t i=0; i<8; i++)
		{
			temptemp=0;
			for(uint8_t p=0; p<5; p++)
			{
				temptemp+=bufforval[i][p];
			}
			temptemp=temptemp/5;

			*ka=temptemp;
			ka++;
		}

		timervalues=200; //pobranie wartosci co 20ms
	}
}

//Funkcja przeliczajaca aktualne wartosci
void calculatevalues(uint16_t *s, uint16_t *voltagemeas1, uint16_t *voltagemeas2, uint16_t *voltagemeas3, uint16_t *voltagemeas4,
		             uint16_t *currentmeas1, uint16_t *currentmeas2, uint16_t *currentmeas3, uint16_t *currentmeas4)
{
	 uint16_t *k;
	 uint32_t temptemp=0;
	 uint16_t tempvalue[8];

	 k=s;

	 for(uint8_t i=0; i<4; i++)
	 {
		 temptemp=0;
		 temptemp=(*k)*refvoltage;
		 tempvalue[i]=temptemp/4095;
		 k++;
	 }
	 for(uint8_t p=0; p<4; p++)
	 {
			 temptemp=0;
			 temptemp=(*k)*refcurrent;
			 tempvalue[p+4]=temptemp/4095;
			 k++;
	 }

	 *voltagemeas1=tempvalue[0];
	 *voltagemeas2=tempvalue[1];
	 *voltagemeas3=tempvalue[2];
	 *voltagemeas4=tempvalue[3];
	 *currentmeas1=tempvalue[4];
	 *currentmeas2=tempvalue[5];
	 *currentmeas3=tempvalue[6];
	 *currentmeas4=tempvalue[7];

	 if(*voltagemeas1>compvoltage) *voltagemeas1=0;
	 if(*voltagemeas2>compvoltage) *voltagemeas2=0;
	 if(*voltagemeas3>compvoltage) *voltagemeas3=0;
	 if(*voltagemeas4>compvoltage) *voltagemeas4=0;

	 if(*currentmeas1<compcurrent) *currentmeas1=0;
	 if(*currentmeas2<compcurrent) *currentmeas2=0;
	 if(*currentmeas3<compcurrent) *currentmeas3=0;
	 if(*currentmeas4<compcurrent) *currentmeas4=0;

}

//Funkcja obliczajaca aktualna wartosc ladunku ktory przeplynal
void calculatecharge(void)
{
static uint32_t tempch1=0, tempch2=0, tempch3=0, tempch4=0;

if(currentmeas1>5)
{
	if(!timercharge1)
	{
		if(flagach1)
		{
			flagach1=0;
			tempch1=0;
		}

		timercharge1=10000;

		tempch1+=currentmeas1;
		columbusmeas1=tempch1/3600;

	}
}

if(currentmeas2>5)
{
	if(!timercharge2)
	{
		if(flagach2)
		{
			flagach2=0;
			tempch2=0;
		}

		timercharge2=10000;

		tempch2+=currentmeas2;
		columbusmeas2=tempch2/3600;

	}
}

if(currentmeas3>5)
{
	if(!timercharge3)
	{
		if(flagach3)
		{
			flagach3=0;
			tempch3=0;
		}

		timercharge3=10000;

		tempch3+=currentmeas3;
		columbusmeas3=tempch3/3600;

	}
}

if(currentmeas4>5)
{
	if(!timercharge4)
	{
		if(flagach4)
		{
			flagach4=0;
			tempch4=0;
		}

		timercharge4=10000;

		tempch4+=currentmeas4;
		columbusmeas4=tempch4/3600;

	}
}
}

//Procedura ustawiajaca aktualne wartosci pradow
void setcurrents(uint16_t *current1, uint16_t *current2, uint16_t *current3, uint16_t *current4)
{
	static uint16_t temp1=1000, temp2=1000, temp3=1000, temp4=1000;

	if(temp1 != *current1)
	{
		temp1=*current1;

		valset1=(*current1)*10;
	}
	if(temp2 != *current2)
	{
		temp2=*current2;

		valset2=(*current2)*10;
	}
	if(temp3 != *current3)
	{
		temp3=*current3;

		valset3=(*current3)*10;
	}
	if(temp4 != *current4)
	{
		temp4=*current4;

		valset4=(*current4)*10;
	}

}

//procedura obslugujaca podswietlenie wyswietlacza
void controlbacklight(void)
{
	while(backlight<1000)
	{
		if(backlight<200)
		backlight+=5;
		else backlight+=10;

		while(timerbacklight)
		{

		}
		timerbacklight=250;
    }
}

//Funkcja kontrolujaca porty wyjsciowe
void controlports(uint8_t *ch1, uint8_t *ch2, uint8_t *ch3, uint8_t *ch4)
{
	static uint8_t tch1=0xff, tch2=0xff, tch3=0xff, tch4=0xff;

	if(tch1!=*ch1)
	{
		tch1=*ch1;
		if(!tch1) uncharge1; else charge1;
	}

	if(tch2!=*ch2)
		{
			tch2=*ch2;
			if(!tch2) uncharge2; else charge2;
		}
	if(tch3!=*ch3)
		{
			tch3=*ch3;
			if(!tch3) uncharge3; else charge3;
		}
	if(tch4!=*ch4)
		{
			tch4=*ch4;
			if(!tch4) uncharge4; else charge4;
		}
}


//Funkcja kontrolujaca paski stanu
void barcontrol(uint16_t *voltage1, uint16_t *voltage2, uint16_t *voltage3, uint16_t *voltage4, uint16_t *bar1, uint16_t *bar2, uint16_t *bar3, uint16_t *bar4)
{
	static uint16_t temp1=1, temp2=0;
	uint32_t bt1, bt2, bt3, bt4;
	static uint16_t volt1=0xffff, volt2=0xffff, volt3=0xffff, volt4=0xffff;

	if(temp1)
	{
		temp1=0;
		temp2=supvoltage-infvoltage;
	}

	if(!timerbar)
	{
		timerbar=300; //odczekanie 30ms

		if(volt1!=*voltage1)
		{
		if(*voltage1>infvoltage)
		{
			if(*voltage1>supvoltage)
				{
				*bar1=100;
				volt1=*voltage1;
				}

			else
			{
				bt1=*voltage1-infvoltage;
				bt1=bt1*100;
				bt1=bt1/temp2;
				if(bt1<*bar1) *bar1-=1; else if(bt1>*bar1) *bar1+=1;
				if(bt1==*bar1) volt1=*voltage1;
			}
		}
		else if(*voltage1<=infvoltage)
			{
			*bar1=0;
			volt1=*voltage1;
			}

		}

		if(volt2!=*voltage2)
		{
			if(*voltage2>infvoltage)
			{
				if(*voltage2>supvoltage)
					{
					*bar2=100;
					volt2=*voltage2;
					}

				else
				{
					bt2=*voltage2-infvoltage;
					bt2=bt2*100;
					bt2=bt2/temp2;
					if(bt2<*bar2) *bar2-=1; else if(bt2>*bar2) *bar2+=1;
					if(bt2==*bar2) volt2=*voltage2;
				}

			}
			else if(*voltage2<=infvoltage)
				{
				*bar2=0;
				volt2=*voltage2;
				}

		}

		if(volt3!=*voltage3)
		{
				if(*voltage3>infvoltage)
				{
					if(*voltage3>supvoltage)
						{
						*bar3=100;
						volt3=*voltage3;
						}

					else
					{
						bt3=*voltage3-infvoltage;
						bt3=bt3*100;
						bt3=bt3/temp2;
						if(bt3<*bar3) *bar3-=1; else if(bt3>*bar3) *bar3+=1;
						if(bt3==*bar3) volt3=*voltage3;
					}

				}
				else if(*voltage3<=infvoltage)
					{
					*bar3=0;
					volt3=*voltage3;
					}

		}

		if(volt4!=*voltage4)
		{
					if(*voltage4>infvoltage)
					{
						if(*voltage4>supvoltage) {
							*bar4=100;
							volt4=*voltage4;
						}

						else
						{
							bt4=*voltage4-infvoltage;
							bt4=bt4*100;
							bt4=bt4/temp2;
							if(bt4<*bar4) *bar4-=1; else if(bt4>*bar4) *bar4+=1;
							if(bt4==*bar4)	volt4=*voltage4;
						}

					}
					else if(*voltage4<=infvoltage) {
						*bar4=0;
						volt4=*voltage4;
					}

		}
	}

}

//Procedura kontrolujaca prace ladowarki - prad
void controlwork(void)
{
	static uint16_t flaga1=0, flaga2=0, flaga3=0, flaga4=0;
	static uint8_t nalad1=0, nalad2=0, nalad3=0, nalad4=0;
	static uint8_t rozlad1=0, rozlad2=0, rozlad3=0, rozlad4=0;
	static uint8_t  tempchunch1=0xff, tempchunch2=0xff, tempchunch3=0xff, tempchunch4=0xff;
	uint8_t temp1, temp2, temp3, temp4;
	static uint8_t ft1=0, ft2=0, ft3=0, ft4=0;
	static uint8_t gg1=0, gg2=0, gg3=0, gg4=0;
	static uint8_t ss1=0, ss2=0, ss3=0, ss4=0;
	static uint8_t kk1=0, kk2=0, kk3=0, kk4=0;
	static uint16_t tt1=0, tt2=0, tt3=0, tt4=0;

	if(tempchunch1!=chunch1)
	{
		tempchunch1=chunch1;
		flaga1=0;
		nalad1=0;
		rozlad1=0;
		ledinfo1off;
		timh1=interval2;
	}

	if(tempchunch2!=chunch2)
		{
			tempchunch2=chunch2;
			flaga2=0;
			nalad2=0;
			rozlad2=0;
			ledinfo2off;
			timh2=interval2;
		}

	if(tempchunch3!=chunch3)
		{
			tempchunch3=chunch3;
			flaga3=0;
			nalad3=0;
			rozlad3=0;
			ledinfo3off;
			timh3=interval2;
		}

	if(tempchunch4!=chunch4)
		{
			tempchunch4=chunch4;
			flaga4=0;
			nalad4=0;
			rozlad4=0;
			ledinfo4off;
			timh4=interval2;
		}

//--------------------------------------------------------
	//CH1
	if(chunch1)
	{
		if(!timh1)
		{
			timh1=interval;

			if((voltagemeas1>supvoltage) && (flaga1<aproxch)) flaga1++;
			else flaga1=0;

			if(flaga1==aproxch)
				{
				nalad1=1;
				ledinfo1on;
				}

			if((voltagemeas1>supersup) && (voltagemeas1<supersup2))
				{
					valuecurrent1=0;
				}

			if(voltagemeas1>supvoltage)
			{
				if(voltagemeas1>supvoltagemeas[0])
					{
					if(supvoltagemeas[0]==0) supvoltagemeas[0]=voltagemeas1;
					else
					{
					ss1++;
					if(ss1>3)
					{
						ss1=0;
						supvoltagemeas[0]=voltagemeas1;
						kk1=1;
						tt1=variabend;
					}
					}
					}
				else
				{
					ss1=0;
					if(supvoltagemeas[0]>voltagemeas1)
					{
						gg1=1;
						temp1=supvoltagemeas[0]-voltagemeas1;
						if(temp1>2)
						{
							if(ft1<endofch) ft1++;
						}
						else if(ft1) ft1=1;
					}
				}
			}
			else
			{
				if(!ft1 && !gg1) supvoltagemeas[0]=0;
				else
				{
										if(supvoltagemeas[0]>voltagemeas1)
										{
											temp1=supvoltagemeas[0]-voltagemeas1;
											if(temp1>2)
											{
												if(ft1<endofch) ft1++;
											}
											else if(ft1) ft1=1;
										}

				}
			}

			if(tt1) tt1--;

			if(!tt1 && kk1)
			{
				kk1=0;
				nalad1=1;
			}

			if(ft1==endofch)
			{
				ft1=0;
				gg1=0;
				nalad1=1;
				ledinfo1on;
				supvoltagemeas[0]=0;
			}

			if(nalad1)
			{
					valuecurrent1=quiescent;

				if(voltagemeas1<=(supvoltage-hysteresis))
					{
					nalad1=0;
					ft1=0;
					gg1=0;
					supvoltagemeas[0]=0;
					kk1=0;
					tt1=0;
					}
				if(voltagemeas1<=(supvoltage-hysteresis3))
					{
					ledinfo1off;
					}
			}
		}
	}
	else
	{
		if(!timh1)
		{
			timh1=interval3;

			if((voltagemeas1<infvoltage) && (flaga1<aproxuch))
				{
				if(currentmeas1>2) flaga1++; else flaga1=0;
				}
			else flaga1=0;

			if(flaga1==aproxuch)
				{
				rozlad1=1;
				ledinfo1on;
				}

			if(rozlad1)
			{
				valuecurrent1=0;

				if(voltagemeas1>=(infvoltage+hysteresis))
					{
					rozlad1=0;
					}
				if(voltagemeas1>=(infvoltage+hysteresis2))
					{
					ledinfo1off;
					}
			}
		}

	}
	//END CH-1
//------------------------------------------------------------------

	//--------------------------------------------------------
		//CH2
		if(chunch2)
		{
			if(!timh2)
			{
				timh2=interval;

				if((voltagemeas2>supvoltage) && (flaga2<aproxch)) flaga2++;
				else flaga2=0;

				if(flaga2==aproxch)
					{
					nalad2=1;
					ledinfo2on;
					}

				if((voltagemeas2>supersup) && (voltagemeas2<supersup2))
				{
					valuecurrent2=0;
				}

				if(voltagemeas2>supvoltage)
				{
					if(voltagemeas2>supvoltagemeas[1])
						{
						if(supvoltagemeas[1]==0) supvoltagemeas[1]=voltagemeas2;
						else
						{
							ss2++;
							if(ss2>3)
							{
								ss2=0;
								supvoltagemeas[1]=voltagemeas2;
								tt2=variabend;
								kk2=1;
							}
						}
						}
					else
					{
						ss2=0;
						if(supvoltagemeas[1]>voltagemeas2)
						{
							gg2=1;
							temp2=supvoltagemeas[1]-voltagemeas2;
							if(temp2>2)
							{
								if(ft2<endofch) ft2++;
							}
							else if(ft2) ft2=1;
						}
					}
				}
				else
				{
					if(!ft2  && !gg2) supvoltagemeas[1]=0;
					else
					{
						if(supvoltagemeas[1]>voltagemeas2)
							{
								temp2=supvoltagemeas[1]-voltagemeas2;
								if(temp2>2)
								{
									if(ft2<endofch) ft2++;
								}
								else if(ft2) ft2=1;
							}
					}
				}

				if(tt2) tt2--;

				if(!tt2 && kk2)
				{
					kk2=0;
					nalad2=1;
				}

				if(ft2==endofch)
					{
								gg2=0;
								ft2=0;
								nalad2=1;
								ledinfo2on;
								supvoltagemeas[1]=0;
					}

				if(nalad2)
				{

						valuecurrent2=quiescent;

					if(voltagemeas2<=(supvoltage-hysteresis))
						{
						nalad2=0;
						ft2=0;
						gg2=0;
						supvoltagemeas[1]=0;
						kk2=0;
						tt2=0;
						}
					if(voltagemeas2<=(supvoltage-hysteresis3))
						{
						ledinfo2off;
						}
				}
			}
		}
		else
		{
			if(!timh2)
			{
				timh2=interval3;

				if((voltagemeas2<infvoltage) && (flaga2<aproxuch))
					{
					if(currentmeas2>2) flaga2++; else flaga2=0;
					}
				else flaga2=0;

				if(flaga2==aproxuch)
					{
					rozlad2=1;
					ledinfo2on;
					}

				if(rozlad2)
				{
					valuecurrent2=0;

					if(voltagemeas2>=(infvoltage+hysteresis))
						{
						rozlad2=0;
						}
					if(voltagemeas2>=(infvoltage+hysteresis2))
						{
						ledinfo2off;
						}
				}
			}

		}
		//END CH-2
	//------------------------------------------------------------------

		//--------------------------------------------------------
				//CH3
				if(chunch3)
				{
					if(!timh3)
					{
						timh3=interval;

						if((voltagemeas3>supvoltage) && (flaga3<aproxch)) flaga3++;
						else flaga3=0;

						if(flaga3==aproxch)
							{
							nalad3=1;
							ledinfo3on;
							}

						if((voltagemeas3>supersup) && (voltagemeas3<supersup2))
							{
							valuecurrent3=0;
							}

						if(voltagemeas3>supvoltage)
						{
							if(voltagemeas3>supvoltagemeas[2])
								{
								if(supvoltagemeas[2]==0) supvoltagemeas[2]=voltagemeas3;
								else
								{
									ss3++;
									if(ss3>3)
									{
										ss3=0;
										supvoltagemeas[2]=voltagemeas3;
										kk3=1;
										tt3=variabend;
									}
								}
								}
							else
							{
								ss3=0;
								if(supvoltagemeas[2]>voltagemeas3)
								{
									gg3=1;
									temp3=supvoltagemeas[2]-voltagemeas3;
									if(temp3>2)
									{
										if(ft3<endofch) ft3++;
									}
									else if(ft3) ft3=1;
								}
							}
						}
						else
						{
							if(!ft3 && !gg3) supvoltagemeas[2]=0;
							else
							{
								if(supvoltagemeas[2]>voltagemeas3)
									{
											temp3=supvoltagemeas[2]-voltagemeas3;
											if(temp3>2)
											{
												if(ft3<endofch) ft3++;
											}
											else if(ft3) ft3=1;

									}
							}
						}

						if(tt3) tt3--;

						if(!tt3 && kk3)
						{
							kk3=0;
							nalad3=1;
						}

						if(ft3==endofch)
									{
										gg3=0;
										ft3=0;
										nalad3=1;
										ledinfo3on;
										supvoltagemeas[2]=0;
									}

						if(nalad3)
						{
								valuecurrent3=quiescent;

							if(voltagemeas3<=(supvoltage-hysteresis))
								{
								nalad3=0;
								ft3=0;
								gg3=0;
								supvoltagemeas[2]=0;
								kk3=0;
								tt3=0;
								}
							if(voltagemeas3<=(supvoltage-hysteresis3))
								{
									ledinfo3off;
								}
						}
					}
				}
				else
				{
					if(!timh3)
					{
						timh3=interval3;

						if((voltagemeas3<infvoltage) && (flaga3<aproxuch))
							{
							if(currentmeas3>2) flaga3++; else flaga3=0;
							}
						else flaga3=0;

						if(flaga3==aproxuch)
							{
							rozlad3=1;
							ledinfo3on;
							}

						if(rozlad3)
						{
							valuecurrent3=0;

							if(voltagemeas3>=(infvoltage+hysteresis))
								{
								rozlad3=0;
								}
							if(voltagemeas3>=(infvoltage+hysteresis2))
														{
														ledinfo3off;
														}
						}
					}

				}
				//END CH-3
			//------------------------------------------------------------------

				//--------------------------------------------------------
								//CH4
								if(chunch4)
								{
									if(!timh4)
									{
										timh4=interval;

										if((voltagemeas4>supvoltage) && (flaga4<aproxch)) flaga4++;
										else flaga4=0;

										if(flaga4==aproxch)
											{
											nalad4=1;
											ledinfo4on;
											}

										if((voltagemeas4>supersup) && (voltagemeas4<supersup2))
											{
												valuecurrent4=0;
											}

										if(voltagemeas4>supvoltage)
										{
											if(voltagemeas4>supvoltagemeas[3])
												{
											if(supvoltagemeas[3]==0) supvoltagemeas[3]=voltagemeas4;
											else
											{
												ss4++;
												if(ss4>3)
												{
													ss4=0;
													 supvoltagemeas[3]=voltagemeas4;
													 kk4=1;
													 tt4=variabend;
												}
											}
												}
											else
											{
												ss4=0;
												if(supvoltagemeas[3]>voltagemeas4)
												{
													gg4=1;
													temp4=supvoltagemeas[3]-voltagemeas4;
													if(temp4>2)
													{
														if(ft4<endofch) ft4++;
													}
													else if(ft4) ft4=1;

												}
											}
										}
										else
										{
										     if(!ft4 && !gg4) supvoltagemeas[3]=0;
										     else
										     {
										    	 if(supvoltagemeas[3]>voltagemeas4)
										    	 		{
										    	 				temp4=supvoltagemeas[3]-voltagemeas4;
										    	 				if(temp4>2)
										    	 				{
										    	 				if(ft4<endofch) ft4++;
										    	 				}
										    	 				else if(ft4) ft4=1;
										    	 		}
										     }
										}

										if(tt4) tt4--;

										if(!tt4 && kk4)
										{
											kk4=0;
											nalad4=1;
										}

										if(ft4==endofch)
													{
														gg4=0;
														ft4=0;
														nalad4=1;
														ledinfo4on;
														supvoltagemeas[3]=0;
													}

										if(nalad4)
										{

												valuecurrent4=quiescent;

											if(voltagemeas4<=(supvoltage-hysteresis))
												{
												nalad4=0;
												ft4=0;
												gg4=0;
												supvoltagemeas[3]=0;
												kk4=0;
												tt4=0;
												}

											if(voltagemeas4<=(supvoltage-hysteresis3))
											{
												ledinfo4off;
											}
										}
									}
								}
								else
								{
									if(!timh4)
									{
										timh4=interval3;

										if((voltagemeas4<infvoltage) && (flaga4<aproxuch))
											{
											if(currentmeas4>2) flaga4++; else flaga4=0;
											}
										else flaga4=0;

										if(flaga4==aproxuch)
											{
											rozlad4=1;
											ledinfo4on;
											}

										if(rozlad4)
										{
											valuecurrent4=0;

											if(voltagemeas4>=(infvoltage+hysteresis))
												{
												rozlad4=0;
												}
											if(voltagemeas4>=(infvoltage+hysteresis2))
																							{
																							ledinfo4off;
																							}
										}
									}

								}
								//END CH-4
							//------------------------------------------------------------------

}



